import base64
import os
from pathlib import Path
from typing import Union, List, Optional

import streamlit.components.v1 as components
import json

_RELEASE = True
RENDERING_EMBED = "legacy_embed"
RENDERING_IFRAME = "legacy_iframe"
RENDERING_UNWRAP = "unwrap"

if not _RELEASE:
    _component_func = components.declare_component(
        "streamlit_pdf_viewer",
        url="http://localhost:3001",
    )
else:
    parent_dir = os.path.dirname(os.path.abspath(__file__))
    build_dir = os.path.join(parent_dir, "frontend/dist")
    _component_func = components.declare_component(
        "streamlit_pdf_viewer",
        path=build_dir
    )


def pdf_viewer(input: Union[str, Path, bytes],
               width: int = None,
               height: int = None,
               key=None,
               annotations: list = (),
               pages_vertical_spacing: int = 2,
               annotation_outline_size: int = 1,
               rendering: str = RENDERING_UNWRAP,
               pages_to_render: List[int] = (),
               render_text: bool = False
               ):
    """
    pdf_viewer function to display a PDF file in a Streamlit app.

    :param input: The source of the PDF file. Accepts a file path, URL, or binary data.
    :param width: Width of the PDF viewer in pixels. Defaults to 700 pixels.
    :param height: Height of the PDF viewer in pixels. If not provided, the viewer show the whole content.
    :param key: An optional key that uniquely identifies this component. Used to preserve state in Streamlit apps.
    :param annotations: A list of annotations to be overlaid on the PDF. Each annotation should be a dictionary.
    :param pages_vertical_spacing: The vertical space (in pixels) between each page of the PDF. Defaults to 2 pixels.
    :param annotation_outline_size: Size of the outline around each annotation in pixels. Defaults to 1 pixel.
    :param rendering: Type of rendering. The default is "unwrap", which unwrap the PDF. Other values are
    :param pages_to_render: Optional list of page numbers to render. If None, all pages are rendered. This allows for selective rendering of pages in the PDF.
    "legacy_iframe" and "legacy_embed" which uses the legacy approach for showing PDF document with streamlit.
    These methods enable the default pdf viewer of Firefox/Chrome/Edge that contains additional features we are still
    working to implement for the "unwrap" method.
    :param render_text: Whether to enable selection of text in the PDF viewer. Defaults to False.

    The function reads the PDF file (from a file path, URL, or binary data), encodes it in base64,
    and uses a Streamlit component to render it in the app. It supports optional annotations and adjustable margins.

    Returns the value of the selected component (if any).
    """

    # Validate width and height parameters
    if width is not None and not isinstance(width, int):
        raise TypeError("Width must be an integer")
    if height is not None and not isinstance(height, int):
        raise TypeError("Height must be an integer or None")
    if not all(isinstance(page, int) for page in pages_to_render):
        raise TypeError("pages_to_render must be a list of integers")

    if type(input) is not bytes:
        with open(input, 'rb') as fo:
            binary = fo.read()
    else:
        binary = input

    if rendering == RENDERING_IFRAME or rendering == RENDERING_EMBED:
        print(f"{RENDERING_IFRAME} and {RENDERING_EMBED} may not work consistently on all browsers "
              f"they might disapper in future releases.")
        if height is None:
            height = "100%"

    base64_pdf = base64.b64encode(binary).decode('utf-8')
    component_value = _component_func(
        binary=base64_pdf,
        width=width,
        height=height,
        key=key,
        default=0,
        annotations=annotations,
        pages_vertical_spacing=pages_vertical_spacing,
        annotation_outline_size=annotation_outline_size,
        rendering=rendering,
        pages_to_render=pages_to_render,
        render_text=render_text
    )
    return component_value


if not _RELEASE:
    with open("resources/test.pdf", 'rb') as fo:
        binary = fo.read()

    with open("resources/annotations.sample.json", 'rb') as fo:
        annotations = json.loads(fo.read())

    viewer = pdf_viewer(
        binary,
        width=800,
        annotations=annotations,
        render_text=True
    )



import os
from pathlib import Path

import pytest
from playwright.sync_api import Page, expect

from tests import ROOT_DIRECTORY
from tests.e2e_utils import StreamlitRunner

BASIC_EXAMPLE_FILE = os.path.join(ROOT_DIRECTORY, "tests", "streamlit_apps", "example_unwrap_no_args.py")


@pytest.fixture(autouse=True, scope="module")
def streamlit_app():
    with StreamlitRunner(Path(BASIC_EXAMPLE_FILE)) as runner:
        yield runner


@pytest.fixture(autouse=True, scope="function")
def go_to_app(page: Page, streamlit_app: StreamlitRunner):
    page.goto(streamlit_app.server_url)
    # Wait for app to load
    page.get_by_role("img", name="Running...").is_hidden()


def test_should_render_template_check_container_size(page: Page):
    expect(page.get_by_text("Test PDF Viewer with no args")).to_be_visible()

    iframe_component = page.locator('iframe[title="streamlit_pdf_viewer.streamlit_pdf_viewer"]').nth(0)
    expect(iframe_component).to_be_visible()

    iframe_box = iframe_component.bounding_box()
    assert iframe_box['width'] > 0
    assert iframe_box['height'] > 0

    iframe_frame = page.frame_locator('iframe[title="streamlit_pdf_viewer.streamlit_pdf_viewer"]')
    pdf_container = iframe_frame.locator('div[id="pdfContainer"]')
    expect(pdf_container).to_be_visible()

    b_box = pdf_container.bounding_box()
    # Since we do not specify the width, we occupy all the available space, which should correspond to the
    # parent element's width of the pdfContainer.
    assert b_box['width'] == iframe_box['width']
    assert b_box['height'] > 0

    pdf_viewer = iframe_frame.locator('div[id="pdfViewer"]')
    expect(pdf_viewer).to_be_visible()

    canvas_list = pdf_viewer.locator("canvas").all()
    assert len(canvas_list) == 8
    for canvas in canvas_list:
        expect(canvas).to_be_visible()

    annotations_locator = page.locator('div[id="pdfAnnotations"]').nth(0)
    expect(annotations_locator).to_be_hidden()

# def test_should_render_template(page: Page):
#     frame_0 = page.frame_locator(
#         'iframe[title="my_component\\.my_component"]'
#     ).nth(0)
#     frame_1 = page.frame_locator(
#         'iframe[title="my_component\\.my_component"]'
#     ).nth(1)
#
#     st_markdown_0 = page.get_by_role('paragraph').nth(0)
#     st_markdown_1 = page.get_by_role('paragraph').nth(1)
#
#     expect(st_markdown_0).to_contain_text("You've clicked 0 times!")
#
#     frame_0.get_by_role("button", name="Click me!").click()
#
#     expect(st_markdown_0).to_contain_text("You've clicked 1 times!")
#     expect(st_markdown_1).to_contain_text("You've clicked 0 times!")
#
#     frame_1.get_by_role("button", name="Click me!").click()
#     frame_1.get_by_role("button", name="Click me!").click()
#
#     expect(st_markdown_0).to_contain_text("You've clicked 1 times!")
#     expect(st_markdown_1).to_contain_text("You've clicked 2 times!")
#
#     page.get_by_label("Enter a name").click()
#     page.get_by_label("Enter a name").fill("World")
#     page.get_by_label("Enter a name").press("Enter")
#
#     expect(frame_1.get_by_text("Hello, World!")).to_be_visible()
#
#     frame_1.get_by_role("button", name="Click me!").click()
#
#     expect(st_markdown_0).to_contain_text("You've clicked 1 times!")
#     expect(st_markdown_1).to_contain_text("You've clicked 3 times!")
#
#
# def test_should_change_iframe_height(page: Page):
#     frame = page.frame_locator('iframe[title="my_component\\.my_component"]').nth(1)
#
#     expect(frame.get_by_text("Hello, Streamlit!")).to_be_visible()
#
#     locator = page.locator('iframe[title="my_component\\.my_component"]').nth(1)
#
#     init_frame_height = locator.bounding_box()['height']
#     assert init_frame_height != 0
#
#     page.get_by_label("Enter a name").click()
#
#     page.get_by_label("Enter a name").fill(35 * "Streamlit ")
#     page.get_by_label("Enter a name").press("Enter")
#
#     expect(frame.get_by_text("Streamlit Streamlit Streamlit")).to_be_visible()
#
#     frame_height = locator.bounding_box()['height']
#     assert frame_height > init_frame_height
#
#     page.set_viewport_size({"width": 150, "height": 150})
#
#     expect(frame.get_by_text("Streamlit Streamlit Streamlit")).not_to_be_in_viewport()
#
#     frame_height_after_viewport_change = locator.bounding_box()['height']
#     assert frame_height_after_viewport_change > frame_height



from pathlib import Path

import setuptools

this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()
bump_version = (this_directory / ".bumpversion.toml").read_text()
current_version_line = [line for line in bump_version.split('\n') if 'current_version' in line][0]
version = current_version_line.split('=')[1].strip().strip('"')

setuptools.setup(
    name="streamlit-pdf-viewer",
    version=version,
    author="Luca Foppiano, Tomoya Mato",
    author_email="lucanoro@duck.com, tomoya.matou@gmail.com",
    description="Streamlit component for PDF visualisation and manipulation",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/lfoppiano/streamlit-pdf-viewer",
    packages=setuptools.find_packages(),
    include_package_data=True,
    python_requires=">=3.7",
    install_requires=[
        "streamlit >= 0.63"
    ],
    extras_require={
        "devel": [
            "wheel"
        ]
    }
)



import os
from math import ceil, floor
from pathlib import Path

import pytest
from playwright.sync_api import Page, expect

from tests import ROOT_DIRECTORY
from tests.e2e_utils import StreamlitRunner

BASIC_EXAMPLE_FILE = os.path.join(ROOT_DIRECTORY, "tests", "streamlit_apps", "example_unwrap_width.py")


@pytest.fixture(autouse=True, scope="module")
def streamlit_app():
    with StreamlitRunner(Path(BASIC_EXAMPLE_FILE)) as runner:
        yield runner


@pytest.fixture(autouse=True, scope="function")
def go_to_app(page: Page, streamlit_app: StreamlitRunner):
    page.goto(streamlit_app.server_url)
    # Wait for app to load
    page.get_by_role("img", name="Running...").is_hidden()


def test_should_render_template_check_container_size(page: Page):
    expect(page.get_by_text("Test PDF Viewer with arguments with specified width")).to_be_visible()

    iframe_component = page.locator('iframe[title="streamlit_pdf_viewer.streamlit_pdf_viewer"]').nth(0)
    expect(iframe_component).to_be_visible()

    iframe_box = iframe_component.bounding_box()
    assert iframe_box['width'] > 0
    assert iframe_box['height'] > 0

    iframe_frame = page.frame_locator('iframe[title="streamlit_pdf_viewer.streamlit_pdf_viewer"]').nth(0)
    pdf_container = iframe_frame.locator('div[id="pdfContainer"]')
    expect(pdf_container).to_be_visible()

    b_box = pdf_container.bounding_box()
    assert floor(b_box['width']) == 400
    assert floor(b_box['height']) == 4221

    pdf_viewer = iframe_frame.locator('div[id="pdfViewer"]')
    expect(pdf_viewer).to_be_visible()

    canvas_list = pdf_viewer.locator("canvas").all()
    assert len(canvas_list) == 8
    for canvas in canvas_list:
        expect(canvas).to_be_visible()

    annotations_locator = page.locator('div[id="pdfAnnotations"]').nth(0)
    expect(annotations_locator).to_be_hidden()
